/*******************************************************************
 * libfaxophone                                                    *
 * Created by Jan-Michael Brummer                                  *
 * All parts are distributed under the terms of GPLv2. See COPYING *
 *******************************************************************/

/**
 * \file phone.c
 * \brief Phone handling functions
 */

#include <string.h>
#include <math.h>
#include <fcntl.h>

#include "faxophone.h"
#include "phone.h"
#include "isdn-convert.h"

#include <speex/speex_echo.h>
#include <speex/speex_preprocess.h>

/** Speex Echo Cancellation (does not work well at the moment) */
//#define SEC 1

#ifdef SEC
static SpeexEchoState *psEchoState = NULL;
#endif
static SpeexPreprocessState *psPreProcessState = NULL;

gpointer phoneInputThread(gpointer pData ) {
	struct sSession *psSession = faxophoneGetSession();
	struct sCapiConnection *psConnection = pData;
	unsigned char anAudioBufferRx[ CAPI_PACKETS * 2 ];
	unsigned char anAudioBuffer[ CAPI_PACKETS * 2 ];
	unsigned int nAudioBufLen;
	_cmsg sCmsg;

	while ( psSession -> nInputThreadState == 1 ) {
		static float fMean = 0.0f;
		short *pnTmp = NULL;
		int nLen = 160;
		
		nLen = sizeof( anAudioBufferRx );
		memset( anAudioBufferRx, 0, nLen );
		nLen = psSession -> psHandlers -> AudioInput( anAudioBufferRx, nLen );

		/* Check if we have some audio data to process */
		if ( nLen >= 0 ) {
			int nIndex;

			/* Remove DC offset */
			pnTmp = ( short * ) anAudioBufferRx;
			for ( nIndex = 0; nIndex < nLen / sizeof( short ); nIndex++ ) {
				fMean = 0.999 * fMean + 0.001 * pnTmp[ nIndex ];
				((short*)anAudioBufferRx)[ nIndex ] = pnTmp[ nIndex ] - ( short ) fMean;
			}

			speex_preprocess( psPreProcessState, ( spx_int16_t * ) anAudioBufferRx, NULL );

			/* convert audio data to isdn format */
			convertAudioToIsdn( psConnection, anAudioBufferRx, nLen, anAudioBuffer, &nAudioBufLen );

			/* Overflow check */
			if ( nAudioBufLen > CAPI_PACKETS ) {
				nAudioBufLen = CAPI_PACKETS;
			}

			isdnLock();
			DATA_B3_REQ( &sCmsg, psSession -> nApplId, 0, psConnection -> nNcci, anAudioBuffer, nAudioBufLen, psSession -> nMessageNumber++, 0 );
			isdnUnlock();
		}
	}

	psSession -> nInputThreadState = 0;

	return NULL;
}

/**
 * \brief Phone transfer routine which accepts incoming data, converts and outputs the audio
 * \param psConnection active capi connection
 * \param sCapiMessage current capi message
 */
void phoneTransfer( struct sCapiConnection *psConnection, _cmsg sCapiMessage ) {
	struct sSession *psSession = faxophoneGetSession();
	_cmsg sCmsg;
	unsigned char anAudioBuffer[ CAPI_PACKETS * 2 ];
	unsigned char anAudioBufferTx[ CAPI_PACKETS * 2 ];
	unsigned char anAudioBufferRx[ CAPI_PACKETS * 2 ];
	unsigned int nLen = DATA_B3_IND_DATALENGTH( &sCapiMessage );
	unsigned int nAudioBufLen;

	if ( psSession -> nInputThreadState == 0 ) {
		int nI = 1;
		int nLen = 320;

		psSession -> nInputThreadState = 1;

#ifdef SEC
		if ( psEchoState == NULL ) {
			psEchoState = speex_echo_state_init( ( nLen ) / ( sizeof( short ) ), 16 * nLen );
		}
#endif

		if ( psPreProcessState == NULL ) {
			psPreProcessState = speex_preprocess_state_init( nLen / sizeof( short ), 8000 );
			speex_preprocess_ctl( psPreProcessState, SPEEX_PREPROCESS_SET_DENOISE, &nI );
		}

		g_thread_create( phoneInputThread, psConnection, FALSE, NULL );
	}

	/* Clear buffer */
	memset( anAudioBufferRx, 0, sizeof( anAudioBufferRx ) );
	memset( anAudioBufferTx, 0, sizeof( anAudioBufferTx ) );
	memset( anAudioBuffer, 0, sizeof( anAudioBuffer ) );

	/* Overflow check */
	if ( nLen > CAPI_PACKETS ) {
		FopDebug( FOP_DEBUG, "Warning: Buffer overflow! %d > %d\n", nLen, CAPI_PACKETS );
		nLen = CAPI_PACKETS;
	}
	
	/* Save incoming data and send response */
	memcpy( anAudioBufferRx, ( uint8_t * ) DATA_B3_IND_DATA( &sCapiMessage ), nLen );
	isdnLock();
	DATA_B3_RESP( &sCmsg, psSession -> nApplId, psSession -> nMessageNumber++, psConnection -> nNcci, DATA_B3_IND_DATAHANDLE( &sCapiMessage ) );
	isdnUnlock();

	/* convert isdn to audio format */
	convertIsdnToAudio( anAudioBufferRx, nLen, anAudioBuffer, &nAudioBufLen );

	/* Send data */
	psSession -> psHandlers -> AudioOutput( anAudioBuffer, nAudioBufLen );
}

/**
 * \brief Dial a given number and set connection type to SESSION_PHONE
 * \param nController capi controller
 * \param pnSource source number (own MSN)
 * \param pnTarget remote number (we want to dial)
 * \param nAnonymous anonymous flag (suppress number)
 * \return seee capiCall
 */
struct sCapiConnection *phoneCall( unsigned char nController, const char *pnSource, const char *pnTarget, unsigned char nAnonymous ) {
	return capiCall( nController, pnSource, pnTarget, nAnonymous, SESSION_PHONE, PHONE_CIP );
}

/**
 * \brief Mute/unmute active capi connection
 * \param psConnection active capi connection
 * \param nMute mute flag
 */
void phoneMute( struct sCapiConnection *psConnection, unsigned char nMute ) {
	/* Just set the flag, the audio input thread handle the mute case */
	psConnection -> nMute = nMute;
}

/**
 * \brief Hold and retrieve active capi connection
 * \param psConnection active capi connection
 * \param nHold hold flag
 */
void phoneHold( struct sCapiConnection *psConnection, unsigned char nHold ) {
	struct sSession *psSession = faxophoneGetSession();
	_cmsg sMessage;
	_cbyte anFac[ 9 ];

	/* Save state */
	psConnection -> nHold = nHold;

	/* Generate facility structure */
	anFac[ 0 ] = 3;
	anFac[ 1 ] = ( _cbyte )( 0x0003 - nHold ) & 0xFF;
	anFac[ 2 ] = ( _cbyte )( ( 0x0003 - nHold ) >> 8 ) & 0xFF;
	anFac[ 3 ] = 0;

	isdnLock();
	if ( nHold == 1 ) {
		/* Hold active connection */
		FACILITY_REQ( &sMessage, psSession -> nApplId, 0, psConnection -> nNcci, 3, ( unsigned char * ) anFac );
	} else {
		/* Retrieve active connection */
		FACILITY_REQ( &sMessage, psSession -> nApplId, 0, psConnection -> nPlci, 3, ( unsigned char * ) anFac );
	}
	isdnUnlock();
}

/**
 * \brief Send DTMF codes on connection
 * \param psConnection active capi connection
 * \param nCode DTMF code
 */
void phoneSendDtmfCode( struct sCapiConnection *psConnection, unsigned char nCode ) {
	capiSendDtmfCode( psConnection, nCode );
}

/**
 * \brief Hangup phone connection
 * \param psConnection active capi connection
 */
void phoneHangup( struct sCapiConnection *psConnection ) {
	if ( psConnection == NULL ) {
		return;
	}
	/* Hangup */
	capiHangup( psConnection );
}

/**
 * \brief Pickup a phone call
 * \param psConnection active capi connection
 * \return see capiPickup
 */
int phonePickup( struct sCapiConnection *psConnection ) {
	if ( psConnection == NULL ) {
		return -1;
	}

	/* Pickup connection and set connection type to SESSION_PHONE */
	return capiPickup( psConnection, SESSION_PHONE );
}
